package com.xhkjedu.sapi.service.itemtool.dict;

import com.xhkjedu.sapi.mapper.itemtool.dict.*;
import com.xhkjedu.sapi.model.itemtool.dict.*;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.sapi.vo.itemtool.dict.WordexamVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName WordService
 * Description TODO
 * Author WN
 * Date 2020/11/21 10:59
 **/
@Service
@Slf4j
public class WordService {
    @Autowired
    private WordMapper wordMapper;
    @Autowired
    private WordtypeMapper wordtypeMapper;
    @Autowired
    private WordtypeMeanMapper wordtypeMeanMapper;
    @Autowired
    private WordphraseMapper wordphraseMapper;
    @Autowired
    private WordphraseMeanMapper wordphraseMeanMapper;
    @Autowired
    private WordphraseMeanExMapper wordphraseMeanExMapper;
    @Autowired
    private WordexamMapper wordexamMapper;
    @Autowired
    private WordexampleMapper wordexampleMapper;
    @Autowired
    private WordexampleExMapper wordexampleExMapper;

    /**
     * 功能描述 保存单词信息
     *
     * @param * @param word
     * @return void
     * @author WN
     * @date 2020/11/21
     */
    @Transactional
    public void saveWord(DWord word) {
        //保存单词信息
        int nowtime = N_Utils.getSecondTimestamp();
        word.setCreatetime(nowtime);
        wordMapper.insertUseGeneratedKeys(word);
        int wordid = word.getId();

        //保存单词类型信息
        saveWordtype(word.getWordtypeList(), wordid, nowtime);

        //保存单词词组相关信息
        saveWordphrase(word.getWordphraseList(), wordid, nowtime);

        //保存单词真题例句
        saveWordexam(word.getWordexamList(), wordid, nowtime);

        //保存单词例句示例信息
        saveWordexample(word.getWordexampleList(), wordid, nowtime);

    }

    //保存单词例句示例信息
    private void saveWordexample(List<DWordexample> wordexampleList, int wordid, int nowtime) {
        if (wordexampleList != null && wordexampleList.size() > 0) {
            for (DWordexample wordexample : wordexampleList) {
                wordexample.setWordid(wordid);
                wordexample.setCreatetime(nowtime);

                wordexampleMapper.insertUseGeneratedKeys(wordexample);
                int exid = wordexample.getId();

                //单词例句示例例句
                List<DWordexampleEx> wordexampleExList = wordexample.getWordexampleExList();
                if (wordexampleExList != null && wordexampleExList.size() > 0) {
                    for (DWordexampleEx wordexampleEx : wordexampleExList) {
                        wordexampleEx.setExid(exid);
                        wordexampleEx.setCreatetime(nowtime);
                    }
                    wordexampleExMapper.insertList(wordexampleExList);
                }
            }
        }
    }

    //保存单词真题例句
    private void saveWordexam(List<DWordexam> wordexamList, int wordid, int nowtime) {
        if (wordexamList != null && wordexamList.size() > 0) {
            for (DWordexam wordexam : wordexamList) {
                wordexam.setWordid(wordid);
                wordexam.setCreatetime(nowtime);
            }
            wordexamMapper.insertList(wordexamList);
        }
    }

    //保存单词词组相关信息
    private void saveWordphrase(List<DWordphrase> wordphraseList, int wordid, int nowtime) {
        if (wordphraseList != null && wordphraseList.size() > 0) {
            for (DWordphrase dphrase : wordphraseList) {
                dphrase.setWordid(wordid);
                dphrase.setCreatetime(nowtime);
                wordphraseMapper.insertUseGeneratedKeys(dphrase);
                int phraseid = dphrase.getId();

                //单词词组释义
                List<DWordphraseMean> wordphraseMeanList = dphrase.getWordphraseMeanList();
                if (wordphraseMeanList != null && wordphraseMeanList.size() > 0) {
                    for (DWordphraseMean phrasemean : wordphraseMeanList) {

                        if(N_Utils.isEmpty(phrasemean.getJx_cn_mean())){
                            break;
                        }
                        phrasemean.setPhraseid(phraseid);
                        phrasemean.setCreatetime(nowtime);

                        wordphraseMeanMapper.insertUseGeneratedKeys(phrasemean);
                        int phrasemeanid = phrasemean.getId();

                        //单词词组释义例句
                        List<DWordphraseMeanEx> wordphraseMeanExList = phrasemean.getWordphraseMeanExList();
                        if (wordphraseMeanExList != null && wordphraseMeanExList.size() > 0) {
                            for (DWordphraseMeanEx phrasemeanex : wordphraseMeanExList) {
                                phrasemeanex.setPhrasemeanid(phrasemeanid);
                                phrasemeanex.setCreatetime(nowtime);
                            }
                            wordphraseMeanExMapper.insertList(wordphraseMeanExList);
                        }
                    }
                }

            }
        }
    }

    //保存单词类型信息
    private void saveWordtype(List<DWordtype> wordtypeList, int wordid, int nowtime) {
        if (wordtypeList != null && wordtypeList.size() > 0) {
            for (DWordtype wtype : wordtypeList) {
                wtype.setWordid(wordid);
                wtype.setCreatetime(nowtime);

                //保存单词类型信息
                wordtypeMapper.insertUseGeneratedKeys(wtype);
                int wordtypeid = wtype.getId();

                //批量保存类型词义
                List<DWordtypeMean> wordtypeMeanList = wtype.getWordtypeMeanList();

                if (wordtypeMeanList != null && wordtypeMeanList.size() > 0) {
                    for (DWordtypeMean wtmean : wordtypeMeanList) {
                        wtmean.setWordtypeid(wordtypeid);
                        wtmean.setCreatetime(nowtime);
                    }
                    wordtypeMeanMapper.insertList(wordtypeMeanList);
                }

            }
        }
    }


    /**
     * 功能描述 查询单词对应信息
     *
     * @param * @param word_name
     * @return com.xhkjedu.model.itemtool.dict.DWord
     * @author WN
     * @date 2020/11/21
     */
    public DWord getWordByName(String word_name) {
        DWord word = null;
        //判断搜索的词是否包含中文 true 包含中文字符 false 不包含中文字符
        boolean ischina = N_Utils.isContainChinese(word_name);
        if (ischina) {
            //如果包含汉字，按照词义搜索单词
            word = wordMapper.getWordByWordmean(word_name);
        } else {
            word = wordMapper.getWordByWordname(word_name);//单词基本信息
        }

        if (word != null && word.getId() > 0) {
            //单词类型
            List<DWordtype> wordtypeList = wordtypeMapper.listWordtypeByWordid(word.getId());

            //单词词组
            List<DWordphrase> wordphrases = wordphraseMapper.listObjsByWordid(word.getId());
            //单词真题例句
            List<DWordexam> wordexamList = wordexamMapper.listObjsByWordid(word.getId());
            WordexamVo examvo = handleWordexam(wordexamList);

            //单词例句示例
            List<DWordexample> wordexampleList = wordexampleMapper.listObjsByWordid(word.getId());
            if (wordexampleList != null && wordexampleList.size() > 0) {
                for (DWordexample we : wordexampleList) {
                    List<DWordexampleEx> wxlist = we.getWordexampleExList();
                    if (wxlist.size() > 20) {
                        List<DWordexampleEx> newlist = wxlist.stream().limit(20).collect(Collectors.toList());
                        we.setWordexampleExList(newlist);
                    }
                }
            }

            word.setWordtypeList(wordtypeList);
            word.setWordphraseList(wordphrases);
            //word.setWordexamList(wordexamList);
            word.setWordexam(examvo);
            word.setWordexampleList(wordexampleList);
        }

        return word;
    }

    private WordexamVo handleWordexam(List<DWordexam> wordexamList) {
        Integer limitnum = 20;
        List<DWordexam> siji = new ArrayList<>();//1四级例句

        List<DWordexam> liuji = new ArrayList<>();//2六级例句

        List<DWordexam> gaokao = new ArrayList<>();//3高考例句

        List<DWordexam> kaoyan = new ArrayList<>();//4考研

        if (wordexamList != null && wordexamList.size() > 0) {
            for (DWordexam we : wordexamList) {
                if (we.getType() == 1) {
                    if (siji.size() < limitnum) {
                        siji.add(we);
                    }
                } else if (we.getType() == 2) {
                    if (liuji.size() < limitnum) {
                        liuji.add(we);
                    }

                } else if (we.getType() == 3) {
                    if (gaokao.size() < limitnum) {
                        gaokao.add(we);
                    }
                } else if (we.getType() == 4) {
                    if (kaoyan.size() < limitnum) {
                        kaoyan.add(we);
                    }

                }
            }
        }

        WordexamVo wordexamVo = new WordexamVo();
        wordexamVo.setSiji(siji);
        wordexamVo.setLiuji(liuji);
        wordexamVo.setGaokao(gaokao);
        wordexamVo.setKaoyan(kaoyan);

        return wordexamVo;
    }


}
