package com.xhkjedu.sapi.service.space;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sapi.mapper.space.ResourceDirectorMapper;
import com.xhkjedu.sapi.mapper.space.ResourceFileMapper;
import com.xhkjedu.sapi.mapper.space.ResourceMapper;
import com.xhkjedu.sapi.mapper.subjectbook.SubjectBookMapper;
import com.xhkjedu.sapi.model.space.TResource;
import com.xhkjedu.sapi.model.space.TResourceDirector;
import com.xhkjedu.sapi.model.space.TResourceFile;
import com.xhkjedu.sapi.model.subjectbook.TSubjectBook;
import com.xhkjedu.sapi.utils.ApiUtil;
import com.xhkjedu.sapi.utils.ConvertUtil;
import com.xhkjedu.sapi.vo.subjectbook.ResourceVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author ywx
 * @classname ResourceService
 * @description 
 * @date 2020/6/5 9:50
 **/
@Service
public class ResourceService {
    @Resource
    private ResourceMapper resourceMapper;
    @Resource
    private ResourceFileMapper resourceFileMapper;
    @Resource
    private ResourceDirectorMapper resourceDirectorMapper;
    @Resource
    private SubjectBookMapper subjectBookMapper;
    @Autowired
    private ConvertUtil convertUtil;

    @Transactional
    public String save(TResource model) {
        model.setReadcount(0);
        model.setDowncount(0);
        model.setResourcestate(1);
        model.setCreatetime(N_Utils.getSecondTimestamp());
        String directorid = model.getDirectorid();
        if (directorid != null && (directorid.equals("0") || directorid.equals(""))) {
            model.setDirectorid(null);
        }
        String resourceid = ApiUtil.getId();
        model.setResourceid(resourceid);
        if (N_Utils.isEmpty(model.getSubjectid())) {//科目为空根据教材id取科目
            TSubjectBook book = subjectBookMapper.selectByPrimaryKey(model.getLsbid());
            model.setSubjectid(book.getSubjectid());
        }

        List<TResourceFile> ziplist = new ArrayList<>();
        if(model.getResourceclass()==7 && model.getCosttime()!=null && model.getCosttime()==7){
            ziplist = JSON.parseArray(model.getPdfurl(),TResourceFile.class);
            for(TResourceFile rf : ziplist){
                rf.setResourceid(resourceid);
            }

            model.setPdfurl(null);
        }

        resourceMapper.insertSelective(model);//保存资源

        List<TResourceDirector> directors = model.getDirectors();
        if(N_Utils.isListEmpty(directors)){//兼容以前添加单个章节模式
            directors = new ArrayList<>();
            TResourceDirector rd = new TResourceDirector();
            rd.setLsbid(model.getLsbid());
            rd.setDirectorid(model.getDirectorid());
            directors.add(rd);
        }
        for(TResourceDirector rd : directors){
            rd.setRdid(ApiUtil.getId());
            rd.setResourceid(resourceid);
            rd.setCreateid(model.getCreateid());
            rd.setCreatetime(model.getCreatetime());
        }

        //如果是分享的压缩包并且已经解压
        if(model.getResourceclass()==7 && model.getCosttime()!=null && model.getCosttime()==7 && ziplist!=null && ziplist.size()>0){
            resourceFileMapper.insertList(ziplist);//批量保存压缩文件的集合
        }

        resourceMapper.batchSaveResourceDirector(directors);//保存资源教材章节关系

        convertUtil.convertResource(model);
        return resourceid;
    }
    //获取详情
    public ResourceVo findById(String resourceid) {
        ResourceVo resource = resourceMapper.findById(resourceid);
        convertUtil.getResourceUrl(resource);
        return resource;
    }

    //分享
    public String share(TResource model) {
        String resourceid = model.getResourceid();

        //判断该资源是否已经分享过该章节，如果已经分享过直接返回
        Integer count = resourceMapper.getLevelResourceForxb(model);
        if(count>0){
            return "99";
        }else{
            return saveShare(model, resourceid);
        }
    }

    /**
     * 分享资源保存
     * @Param [model, resourceid]
     * @Author ywx
     * @Date 2020/7/1 10:18
     * @return java.lang.Integer
     **/
    private String saveShare(TResource model, String resourceid) {
        if (N_Utils.isNotEmpty(resourceid)) {
            ResourceVo r = findById(resourceid);
            model.setResourcename(r.getResourcename());
            model.setResourceclass(r.getResourceclass());
            model.setResourceurl(r.getResourceurl());
            model.setSuffix(r.getSuffix());
            model.setResourcesize(r.getResourcesize());
            model.setResourcecover(r.getResourcecover());
            model.setConverted(r.getConverted());
            model.setPdfurl(r.getPdfurl());
            model.setComm(r.getComm());
            model.setDuration(r.getDuration());
            model.setMp4code(r.getMp4code());
            if(model.getLevel()==1){
                model.setBelong(3);
            }else{
                model.setBelong(r.getBelong());
            }

            if (model.getResourcetype() == null) {
                model.setResourcetype(r.getResourcetype());
            }

            if(r.getResourceclass()==7 && N_Utils.isNotEmpty(r.getPdfurl())){
                //压缩包，用costtime替代分享时的压缩包，设置为7
                model.setCosttime(7);
            }
        }
        save(model);
        //需要条用资源内保存方法
        return "";
    }


    /**
     *功能描述 我的空间列表
     * @author WN
     * @date 2022/2/22
     * @param  * @param resource
     * @return java.util.List<com.xhkjedu.sapi.model.space.TResource>
     */
    public List<TResource> listMyResource(TResource resource) {
        return resourceMapper.listMyResource(resource);
    }
}
