package com.xhkjedu.sstudy.service.subjectbook;


import com.xhkjedu.sstudy.mapper.subjectbook.DirectorMapper;
import com.xhkjedu.sstudy.model.subjectbook.TDirector;
import com.xhkjedu.sstudy.utils.StudyUtil;
import com.xhkjedu.utils.N_Utils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName DirectorService
 * Description TODO
 * Author WN
 * Date 2020/6/3 15:59
 **/
@Service
public class DirectorService {
    @Autowired
    private DirectorMapper directorMapper;

    /**
     *功能描述 新增
     * @author WN
     * @date 2020/6/3
     * @param  * @param director
     * @return java.lang.Integer
     */
    public Integer addDirector(TDirector director){
        String id = StudyUtil.getId();
        director.setDirectorid(id);
        if ("".equals(director.getDirectorpid())) {
            director.setDirectorpid(null);
        }
        director.setCreatetime(N_Utils.getSecondTimestamp());
        return directorMapper.insertSelective(director);
    }

    /**
     * 添加并返回主键
     * @Param [director]
     * @Author ywx
     * @Date 2020/7/6 15:00
     * @return java.lang.Integer
     **/
    public String addAndReturnKey(TDirector director){
        String id = StudyUtil.getId();
        director.setDirectorid(id);
        director.setCreatetime(N_Utils.getSecondTimestamp());
        directorMapper.insertSelective(director);
        return id;
    }

    /**
     *功能描述 修改
     * @author WN
     * @date 2020/6/3
     * @param  * @param director
     * @return java.lang.Integer
     */
    public Integer updateDirector(TDirector director){
        director.setCreatetime(N_Utils.getSecondTimestamp());
        return directorMapper.updateByPrimaryKeySelective(director);
    }

    /**
     *功能描述  删除
     * @author WN
     * @date 2020/6/3
     * @param  * @param director
     * @return java.lang.Integer
     */
    public Integer delDirector(TDirector director){
        return directorMapper.deleteByPrimaryKey(director.getDirectorid());
    }

    /**
     *功能描述  章节树形结构
     * @author WN
     * @date 2020/6/3
     * @param  * @param director
     * @return java.util.List<com.xhkjedu.model.subjectbook.TDirector>
     */
    public List<TDirector> lisTDirectors(TDirector director){
        List<TDirector> lst = directorMapper.listDirectors(director);

        List<TDirector> directors = new ArrayList<>();
        for (TDirector d:lst){
            if(d.getDirectorpid() == null){
                List<TDirector> childDirectors = listChildedirector(d,lst);
                d.setChildren(childDirectors);
                directors.add(d);
            }
        }

        return directors;
    }

    //获取子章节
    private List<TDirector> listChildedirector(TDirector director,List<TDirector> directors){
        List<TDirector> rtndirectors = new ArrayList<>();
        for(TDirector d : directors){
            if(director.getDirectorid().equals(d.getDirectorpid())){
                List<TDirector> childDirectors = listChildedirector(d,directors);
                d.setChildren(childDirectors);
                rtndirectors.add(d);
            }
        }
        return rtndirectors;
    }

    /**
     * 教师收藏教材下的章节数量
     * @Param [teacherid, lsbid]
     * @Author ywx
     * @Date 2020/7/8 14:49
     * @return java.lang.Integer
     **/
    public Integer countByTeacherIdLsbId(Integer teacherid, String lsbid) {
        return directorMapper.countByTeacherIdLsbId(teacherid, lsbid);
    }

    //查找使用章节的模块
    public String findUseModule(String directorid) {
        String useModule = directorMapper.findUseModule(directorid);
        if (useModule.startsWith(",")) {
            useModule = useModule.substring(1);
        }
        return useModule;
    }
}
